<?php
/**
 * The main function to send notifications
 * @param  array $pushover_args Holds the API Arguments used for Pushover.net
 * @return void
 */
function ckpn_send_notification( $pushover_args ) {
	$logentry = '';
	$invalid = false;
	$options = ckpn_get_options();

	$defaults = array(
		'token' => ckpn_get_application_key_by_id(),
		'user' => NULL );

	$api_args = wp_parse_args( $pushover_args, $defaults );

	// Check that we have the required elements
	if ( !isset( $api_args['title'] ) ) {
		$logentry .= ckpn_log_entry_format( '****** Missing Title ******' );
		$invalid = true;
	}

	if ( !isset( $api_args['message'] ) ) {
		$logentry .= ckpn_log_entry_format( '****** Missing Message ******' );
		$invalid = true;
	}

	if ( $api_args['user'] == NULL ) { // Use the default key if none was provided
		$api_args['user'] = $options['api_key'];
	}

	if ( !isset( $api_args['user'] ) ) {
		$logentry .= ckpn_log_entry_format( '****** Missing User Key ******' );
		$invalid = true;
	}

	if ( !isset( $api_args['token'] ) ) {
		$logentry .= ckpn_log_entry_format( '****** Missing Application Key ******' );
		$invalid = true;
	}

	if ( !$invalid ) {
		//Clean house a bit to meet the guidelines of the api
		//https://pushover.net/api#limits
		// Title + Message Limit: 512 Characters
		if ( strlen( $api_args['message'] ) + strlen( $api_args['title'] ) > 512 ) {
			$total_len = strlen( $api_args['message'] ) + strlen( $api_args['title'] );
			$overage = $total_len - 520;
			$api_args['message'] = substr( $api_args['message'], 0, -$overage );
			$last_space = strrpos( $api_args['message'], ' ' );
			$api_args['message'] = substr( $api_args['message'], 0, $last_space ) . '&hellip;';

			$logentry .= ckpn_log_entry_format( 'Title & Message too long, truncating message' );
		}
		// Let's also get entities ready
		$api_args['title']  = html_entity_decode( $api_args['title'], ENT_QUOTES, 'UTF-8' );
		$api_args['message'] = html_entity_decode( $api_args['message'], ENT_QUOTES, 'UTF-8' );

		// URL limit: 500 Characters
		if ( isset( $api_args['url'] ) && strlen( $api_args['url'] ) > 500 ) {
			unset ( $api_args['url'], $api_args['url_title'] );
			$logentry .= ckpn_log_entry_format( 'URL was over 500 characters. Removed URL and URL Title' );
		}

		// URL Title limit: 50 Characters
		if ( isset( $api_args['url_title'] ) && strlen( $api_args['url_title'] ) > 50 ) {
			unset ( $api_args['url_title'] );
			$logentry .= ckpn_log_entry_format( 'URL Title was over 50 characters. Removed URL Title' );
		}

		// Crate the request
		$body = '';
		foreach ( $api_args as $key => $value ) {
			$body .= ( $body == '' ? '' : '&' );
			$body .= urlencode( $key ) . '=' . urlencode( $value );
		}

		$req_args = array( 'body'=>$body );

		if ( $options['sslverify'] == false )
			$req_args['sslverify'] = false;

		if ( $options['logging'] == false )
			$req_args['blocking'] = false;

		// Where the magic happens
		$response = wp_remote_post( 'https://api.pushover.net/1/messages.json', $req_args );

		// From here on out it's just managing logging (if enabled)
		if ( $options['logging'] == 1 ) {
			// If the API Fails for some reason send it via the email system of Pushover
			// This can only be done with logging enabled as it would require a blocking request
			if ( is_wp_error( $response ) ) { 
				$to = $user_key.'@api.pushover.net';
				if ( wp_mail( $to, $subject, $message ) ) {
					$logentry .= ckpn_log_entry_format( 'Email - ' . $api_args['user'] . ' - Subject: ' . $api_args['title'] );
				} else {
					$logentry .= ckpn_log_entry_format( '******Email Notification Failed******' );
				}
				$logentry .= ckpn_log_entry_format( '******API Notification Failed******' );
			} elseif ( $response['response']['code'] == 500 ) {
				$logentry .= ckpn_log_entry_format( 'API Response - Internal Server Error 500' );
				$logentry .= ckpn_log_entry_format( 'Typically occurs with non-entitied characters' );
			} else {
				$logentry .= ckpn_log_entry_format( 'API - ' . $api_args['user'] . ' | ' . $api_args['token'] . ' - ' . $api_args['title'] );
			}
		}
	}

	if ( $options['logging'] == 1 ) {
		$current_logs = get_option( 'ckpn_logs' );
		$new_logs = $logentry . $current_logs;

		$logs_array = explode( "\n", $new_logs );
		if ( count( $logs_array ) > 100 ) {
			$logs_array = array_slice( $logs_array, 0, 100 );
			$new_logs = implode( "\n", $logs_array );
		}

		update_option( 'ckpn_logs', $new_logs );
	}
}

/**
 * Send Notificaitons for plugin/theme upgrades
 * @return void
 */
function ckpn_plugin_update_checks() {
	$options = ckpn_get_options();
	require_once ( ABSPATH . 'wp-admin/includes/update.php' );
	require_once ( ABSPATH . 'wp-admin/includes/admin.php' );

	// Force an update check
	wp_plugin_update_rows();
	$plugin_updates = get_site_transient( 'update_plugins' );
	$plugin_count 	= count( $plugin_updates->response );

	wp_theme_update_rows();
	$theme_updates  = get_site_transient( 'update_themes' );
	$theme_count	=  count( $theme_updates->response );

	if ( empty( $plugin_count ) && empty( $theme_count ) )
		return false;

	$title 		 = apply_filters( 'ckpn_plugin_update_title', get_bloginfo( 'name' ) . ': ' . __( 'Updates Availalbe', CKPN_CORE_TEXT_DOMAIN ) );

	$message  	 = '';
	$plugin_text = _n( 'Plugin has', 'Plugins have', $plugin_count, CKPN_CORE_TEXT_DOMAIN );
	$message 	.= ( !empty( $plugin_count ) ) ? sprintf( __( '%d %s updates', CKPN_CORE_TEXT_DOMAIN ), $plugin_count, $plugin_text ) . "\n" : '';

	$theme_text  = _n( 'Theme has', 'Themes have', $theme_count, CKPN_CORE_TEXT_DOMAIN );
	$message 	.= ( !empty( $theme_count ) ) ? sprintf( __( '%d %s updates', CKPN_CORE_TEXT_DOMAIN ), $theme_count, $theme_text ) : '';
	
	$priority 	 = '1';
	$url 		 = admin_url( 'update-core.php' );
	$url_title	 = 'Update Now';

	$args = array( 'title' => $title, 'message' => $message, 'priority' => $priority, 'url' => $url, 'url_title' => $url_title );

	if ( $options['multiple_keys'] )
		$args['token'] = ckpn_get_application_key_by_setting( 'plugin_updates' );

	ckpn_send_notification( $args );
}

/**
 * Send Notifications for new user registrations
 * @param  int $user_id User ID of the new registered account, passed from the action
 * @return void
 */
function ckpn_user_registration( $user_id ) {
	$options = ckpn_get_options();
	$title = apply_filters( 'ckpn_newuser_subject', get_bloginfo( 'name' ) . ': ' . __( 'New User', CKPN_CORE_TEXT_DOMAIN ) , $user_id);
	
	$user_data = get_userdata( $user_id );
	$message = apply_filters( 'ckpn_newuser_message', sprintf( __( '%s created an account.', CKPN_CORE_TEXT_DOMAIN ), $user_data->user_login ), $user_id, $user_data->user_login);
	
	if ( $title === false || $message === false )
		return;

	$args = apply_filters( 'ckpn_newuser_args', array( 'title' => $title, 'message' => $message ), $user_id );

	if ( $options['multiple_keys'] )
		$args['token'] = ckpn_get_application_key_by_setting( 'new_user' );

	ckpn_send_notification( $args );
}

/**
 * Send Notifications for new comments
 * @param  int $comment_id The ID of the newly submitted comment
 * @return void
 */
function ckpn_new_comment( $comment_id ) {
	$options = ckpn_get_options();
	$comment_data = get_comment( $comment_id );

	// This is not the comment we're looking for. Move Along.
	if ( $comment_data->comment_approved == 'spam' )
		return;

	switch ( $comment_data->comment_type ) {
		case 'pingback':
			$comment_type = __( 'pingback', CKPN_CORE_TEXT_DOMAIN );
			break;
		case 'trackback':
			$comment_type = __( 'trackback', CKPN_CORE_TEXT_DOMAIN );
			break;
		case 'comment':
		default:
			$comment_type = __( 'comment', CKPN_CORE_TEXT_DOMAIN );
			if ( $comment_data->comment_approved != 0 ){
				$url = get_comment_link( $comment_data );
				$url_title = 'View Comment';
			}
			break;
	}

	$title = apply_filters( 'ckpn_newcomment_subject', get_bloginfo( 'name' ) . ': ' . ucfirst( $comment_type ) );
	$post_data = get_post( $comment_data->comment_post_ID );
	$message = apply_filters( 'ckpn_newcomment_message', sprintf( __( 'by %1$s on %2$s', CKPN_CORE_TEXT_DOMAIN ), $comment_data->comment_author, $post_data->post_title ) );

	// Notify the Admin User
	$args = array( 'title' => $title, 'message' => $message );

	if ( isset( $url ) && isset( $url_title ) ) {
		$args['url'] = $url;
		$args['url_title'] = $url_title;
	}

	if ( $options['multiple_keys'] )
		$args['token'] = ckpn_get_application_key_by_setting( 'new_comment' );

	ckpn_send_notification( $args );

	// Check if we should notify the author as well
	if ( $options['notify_authors'] ) {
		$author_user_key = get_user_meta( $post_data->post_author, 'ckpn_user_key', true );
		if ( $author_user_key != '' && $author_user_key != $options['api_key'] ) { // Only send if the user has a key and it's not the same as the admin key
			// Notify the Author their post has a comment
			$args['user'] = $author_user_key;

			if ( $options['multiple_keys'] )
				$args['token'] = ckpn_get_application_key_by_setting( 'notify_authors' );

			ckpn_send_notification( $args );
		}
	}
}

/**
 * Send notifications for lost password requests
 * @return void
 */
function ckpn_lost_password_request() {
	if ( !empty( $_POST['user_login'] ) ) {
		if ( strpos( $_POST['user_login'], '@' ) ) {
			$user_data = get_user_by( 'email', trim( $_POST['user_login'] ) );
		} else {
			$login = trim( $_POST['user_login'] );
			$user_data = get_user_by( 'login', $login );
		}
	}

	if ( !empty( $user_data ) ) {
		$user_pushover_key = get_user_meta( $user_data->data->ID, 'ckpn_user_key', true );
		if ( $user_pushover_key != '' ) {
			$options = ckpn_get_options();
			$title = apply_filters( 'ckpn_password_request_subject', get_bloginfo( 'name' ) . ': ' . __( 'Password Reset Request', CKPN_CORE_TEXT_DOMAIN ) );
			$message = apply_filters( 'ckpn_password_request_message', sprintf( __( 'A password reset request was made for your account. If this was not you pelase verify your account is secure.', CKPN_CORE_TEXT_DOMAIN ), $user_data->data->user_login ) );

			$args = array( 'title' => $title, 'message' => $message, 'user' => $user_pushover_key, 'priority' => 1 );

			if ( $options['multiple_keys'] )
				$args['token'] = ckpn_get_application_key_by_setting( 'password_reset' );

			ckpn_send_notification( $args );
		}
	}
}